<?php

namespace Drupal\acquia_contenthub\EventSubscriber\UnserializeContentField;

use Drupal\acquia_contenthub\AcquiaContentHubEvents;
use Drupal\acquia_contenthub\Event\UnserializeCDFEntityFieldEvent;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;

/**
 * Link Field Unserializer.
 *
 * This class handles the unserialization of menu_link entities.
 */
class LinkFieldUnserializer implements EventSubscriberInterface {
  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents() {
    $events[AcquiaContentHubEvents::UNSERIALIZE_CONTENT_ENTITY_FIELD] = ['onUnserializeContentField', 10];
    return $events;
  }

  /**
   * On unserialize field event function.
   * 
   * Handles the unserialization of menu_link entities.
   *
   * @param \Drupal\acquia_contenthub\Event\UnserializeCDFEntityFieldEvent $event
   *   The unserialize event.
   */
  public function onUnserializeContentField(UnserializeCDFEntityFieldEvent $event) {
    // Get field meta data.
    $meta = $event->getFieldMetadata();
    
    // Make sure the field type is link and uri type is set.
    if ($meta['type'] != 'link' || !isset($meta['uri_type'])) {
      return;
    }

    // Get field and init values array to set later.
    $field = $event->getField();
    $values = [];

    // Return early if no attr values are set.
    if (empty($field['value'])) {
      return;
    }

    // Loop through field values.
    foreach ($field['value'] as $langcode => $value) {
      // Uri type switch.
      switch($meta['uri_type']){
        // Entity link.
        case 'entity':
          // Get the entity from event stack.
          $uri_entity = $event->getStack()->getDependency($value)->getEntity();
          
          // Construct the entity link.
          // Format: entity:<ENT_TYPE>/<ENT_ID>
          $entity_link = "entity:{$uri_entity->getEntityTypeId()}/{$uri_entity->id()}";

          // Set entity link as target.
          $values[$langcode][$event->getFieldName()] = $entity_link;
          break;

        // External and internal links.
        case 'external':
          $values[$langcode][$event->getFieldName()] = $value;
          break;
        case 'internal':
          $values[$langcode][$event->getFieldName()] = "internal:$value";
          break;
      }
    }
    // Set updated event values.
    $event->setValue($values);
    $event->stopPropagation();
  }
}
